"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "cleanup", {
  enumerable: true,
  get: function get() {
    return _reactTestingLibrary.cleanup;
  }
});
exports.useHook = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactTestingLibrary = require("react-testing-library");

var _invariant = _interopRequireDefault(require("invariant"));

var _uuidV = _interopRequireDefault(require("uuid-v4"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var useHook = function useHook(hook) {
  for (var _len = arguments.length, props = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    props[_key - 1] = arguments[_key];
  }

  var context = {
    id: (0, _uuidV.default)(),
    resolveComponent: function resolveComponent(Component) {
      return Component;
    },
    rendered: false,
    props: props
  };

  var HookHarness = function HookHarness() {
    context.currentValue = hook.apply(void 0, context.props);
    return _react.default.createElement("div", {
      "data-testid": context.id
    });
  };

  var renderHook = function renderHook() {
    var _render = (0, _reactTestingLibrary.render)(context.resolveComponent(_react.default.createElement(HookHarness, null))),
        queryByTestId = _render.queryByTestId,
        rerender = _render.rerender;

    var container = queryByTestId(context.id);
    (0, _invariant.default)(container !== null, 'Failed to render wrapper component');
    context.rendered = true;

    context.rerender = function () {
      return rerender(context.resolveComponent(_react.default.createElement(HookHarness, null)));
    };
  };

  var getCurrentValue = function getCurrentValue() {
    if (!context.rendered) {
      renderHook();
    } else {
      context.rerender();
    }

    return context.currentValue;
  };

  var setProps = function setProps() {
    for (var _len2 = arguments.length, newProps = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      newProps[_key2] = arguments[_key2];
    }

    context.props = newProps;
  };

  var addContextProvider = function addContextProvider(ContextProvider, contextProps) {
    var Provider = ContextProvider.Provider || ContextProvider;
    var resolveComponent = context.resolveComponent;

    var updateContext = function updateContext(newContextProps) {
      contextProps = newContextProps;
    };

    context.resolveComponent = function (Component) {
      return _react.default.createElement(Provider, contextProps, resolveComponent(Component));
    };

    return {
      updateContext: updateContext
    };
  };

  var flushEffects = function flushEffects(minTimes, maxTimes) {
    if (minTimes === void 0) {
      minTimes = 1;
    }

    if (maxTimes === void 0) {
      maxTimes = Math.max(minTimes + 1, 100);
    }

    (0, _invariant.default)(minTimes > 0, "minTimes (" + minTimes + ") must be a positive number");
    (0, _invariant.default)(maxTimes > 0, "maxTimes (" + maxTimes + ") must be a positive number");
    (0, _invariant.default)(minTimes <= maxTimes, "maxTimes (" + maxTimes + ") must be less than maxTimes (" + maxTimes + ")");
    var lastValue;
    var currentValue;
    var flushCount = 0;

    while ((currentValue !== lastValue || flushCount < minTimes) && flushCount < maxTimes) {
      lastValue = currentValue;
      currentValue = getCurrentValue();
      flushCount++;
    }

    (0, _invariant.default)(flushCount < maxTimes, "Hook values have not resolved after flushing " + maxTimes + " times");
  };

  return {
    getCurrentValue: getCurrentValue,
    getCurrentValues: getCurrentValue,
    flushEffects: flushEffects,
    setProps: setProps,
    addContextProvider: addContextProvider
  };
};

exports.useHook = useHook;